//*************************************************************
// DEMO.C 
// 在LCD第一行顯示 “Welcome to PIC” 
// 在LCD第二行顯示 “Micro-Controller” 
//*************************************************************
#include<xc.h>//納入微控制器定義檔
#include"evm_lcd.h"//納入LCD函式庫定義檔
#include<stdio.h>//納入標準函式庫函式原型名稱定義
#include<string.h>//納入標準函式庫函式原型名稱定義
#include<ctype.h>//納入字元符號函式庫函式原型名稱定義

/* Revsesion Original form C18 to XC8 1.34 */
/* Modified that Include, Interrupt Definition and btoa -> sprintf */

// 設定微處理器相關結構位元
#pragma config OSC=HS, BOREN=OFF, BORV = 2, PWRT=ON, WDT=OFF, LVP=OFF
//, CCP2MX = PORTBE

// 定義時序頻率
#define OSC_CLOCK 10

// 宣告時間延遲函式原型
void delay_ms(long);
void rot_right(void);
void rot_left(void);
void put_Num_LCD(unsigned char);

// 宣告時間相關變數並初始化為0
unsigned char hour = 0, min = 0, sec = 0;
unsigned char DisplayString[20];
// 狀態旗標變數

struct Flag
{
    unsigned update : 1; // One Second Passed
    unsigned TxD : 1; // UART Tx Continue
} FLAGbits;

// 宣告TIMER1計時器中斷執行程式
void timer1_isr(void);
void Rx_isr(void);

void interrupt high_interrupt(void)
{
    // timer1_isr
    PIR1bits.TMR1IF = 0; // 清除中斷旗標
    WriteTimer1(0x8000); // 設定計時器初始值
    FLAGbits.update = 1; // 設定更新旗標
}

void interrupt low_priority low_interrupt(void)
{
    // Rx_isr
    PIR1bits.RCIF = 0; // 清除中斷旗標
    ReadUSART(); // 讀取USART資料位元組
    FLAGbits.TxD = 1; // 設定旗標
}

#define I2C_TCN75A_address 0x98 // TCN75A I2C Device Address

void main()
{
    unsigned char Temp[2];
    char char_str[5] = {0, 0, 0, 0, 0};

    FLAGbits.update = 0;
    FLAGbits.TxD = 0;

    OpenLCD(); // 初始化LCD模組

    WriteCmdLCD(0x01); // 清除LCD顯示資料
    LCD_Set_Cursor(0, 0); // 顯示位置回至第0行第0格
    delay_ms(1); // 時間延遲
    putrsLCD("Welcome to PIC"); // 顯示資料
    LCD_Set_Cursor(1, 0); // 顯示位置調至第1行第0格
    delay_ms(1); // 時間延遲
    putrsLCD("Micro-Controller"); // 顯示資料

    delay_ms(1000);

    PORTD = 0x01; // 將PORTD設定點亮LED0
    TRISD = 0; // 將TRISD設為0，PORTD設定為輸出
    ADCON1 = 0x0C; // 設定AN0~2為類比輸入
    TRISAbits.TRISA4 = 1; // 設定RA4為數位輸入腳位
    TRISBbits.TRISB0 = 1; // 設定RB0為數位輸入腳位

    // 開啟類比訊號轉換模組
    OpenADC(ADC_FOSC_32 & ADC_LEFT_JUST & ADC_20_TAD,
            ADC_CH0 & ADC_INT_OFF & ADC_VREFPLUS_VDD &
            ADC_VREFMINUS_VSS, 12);

    TRISCbits.TRISC2 = 0; // 設定CCP1為輸出
    TRISBbits.TRISB3 = 0; // 設定CCP2為輸出
    OpenPWM1(0x9B); // 設定CCP1為PWM功能與時脈周期
    OpenPWM2(0x9B); // 設定CCP2為PWM功能與時脈周期
    OpenTimer2(TIMER_INT_OFF & T2_PS_1_4 & T2_POST_1_1); //開啟TIMER2
    /*
    OpenI2C(MASTER, SLEW_ON);// 初始化I2C模組
    SSPADD = 0x19; //400Khz Baud clock(0x19) @10MHz
    //100khz Baud clock(0x3F) @10MHz

    EEByteWrite(I2C_TCN75A_address, 0x01, 0x00); // 初始化TCN75A模組為9bit模式
    char_str[0]=34;//設定溫度警示上限=char_str[0]+char_str[1]/256
    char_str[1]=128; //僅bit7有作用
    EEPageWrite(I2C_TCN75A_address, 0x03, char_str); // 設定溫度警示上限
    char_str[0]=33;//設定溫度警示解除下限=char_str[0]+char_str[1]/256
    char_str[1]=128; //僅bit7有作用
    EEPageWrite(I2C_TCN75A_address, 0x02, char_str); // 設定溫度警示解除下限
     */
    OpenUSART(USART_TX_INT_OFF & //開啟USART通訊介面
              USART_RX_INT_ON & //設定為8-N-1-9600模式
              USART_ASYNCH_MODE & //開啟資料接收中斷功能
              USART_EIGHT_BIT &
              USART_CONT_RX &
              USART_BRGH_HIGH,
              64);

    // The Baud Rate is Fosc/(64*(spbrg+1)); for Low Speed
    // The Baud Rate is Fosc/(16*(spbrg+1)); for High Speed
    // 64 and High is 9600
    PIR1bits.RCIF = 0; // 清除中斷旗標
    IPR1bits.RCIP = 0; // 設定為低優先中斷

    // 初始化計時器Timer1
    OpenTimer1(TIMER_INT_ON &
               T1_16BIT_RW &
               T1_SOURCE_EXT &
               T1_PS_1_1 &
               T1_OSC1EN_ON &
               T1_SYNC_EXT_OFF);
    WriteTimer1(0x8000); //0x8000=1sec @32786Hz Crystal
    PIR1bits.TMR1IF = 0; // 清除中斷旗標
    IPR1bits.TMR1IP = 1; // 設定為高優先中斷

    RCONbits.IPEN = 1; // 開啟中斷優先功能
    INTCONbits.GIEL = 1; // 開啟低優先中斷功能
    INTCONbits.GIEH = 1; // 開啟高優先中斷功能

    LCD_Set_Cursor(1, 0); // 顯示位置回至第0行第0格
    delay_ms(1); // 時間延遲
    putrsLCD("S2:MORE S3:LED "); // 顯示資料

    while (PORTBbits.RB0 == 1)
    {

        delay_ms(100); // 延遲100ms
        if (PORTAbits.RA4 == 0) // 按鍵按下時，向左移動
            rot_left();
        else // 否則，向右移動
            rot_right();
    }

    WriteCmdLCD(0x01); // 清除LCD顯示資料
    delay_ms(1); // 時間延遲
    LCD_Set_Cursor(0, 0); // 顯示位置回至第0行第0格
    delay_ms(1); // 時間延遲
    putrsLCD("V1:??? V2:??? S"); // 顯示資料
    LCD_Set_Cursor(1, 0); // 顯示位置回至第0行第0格
    putrsLCD("0:0:0"); // 初始化LCD

    while (1)
    {
        SetChanADC(ADC_CH0);
        ConvertADC(); // 進行訊號轉換
        while (BusyADC()); // 等待轉換完成
        LCD_Set_Cursor(0, 3); // 顯示位置回至第0行第0格
        put_Num_LCD(ADRESH);
        SetDCPWM1((unsigned int) ADRESH << 2); // 調整PWM工作週期

        SetChanADC(ADC_CH1);
        ConvertADC(); // 進行訊號轉換
        while (BusyADC()); // 等待轉換完成
        LCD_Set_Cursor(0, 10); // 顯示位置回至第0行第0格
        put_Num_LCD(ADRESH);
        SetDCPWM2((unsigned int) ADRESH << 2); // 調整PWM工作週期

        LCD_Set_Cursor(0, 15); // 顯示位置回至第0行第0格
        SetChanADC(ADC_CH2);
        ConvertADC(); // 進行AN2訊號轉換
        while (BusyADC()); // 等待轉換完成
        if (ADRESH == 0) putcLCD('4'); // 根據轉換結果決定觸發之類比按鍵
        else
        {
            if (ADRESH < 128) putcLCD('5');
            else
            {
                if (ADRESH < 170) putcLCD('6');
                else putcLCD(' ');
            }
        }

        if (FLAGbits.update)
        {
            ++sec; // 將秒數遞加
            if (sec >= 60)
            { // 作進位處理
                sec -= 60;
                min++;
            }
            if (min >= 60)
            {
                min -= 60;
                hour++;
            }
            if (hour > 24) hour = 0;
            LCD_Set_Cursor(1, 0); // 調整顯示位址
            delay_ms(1);
            putrsLCD(" ");
            delay_ms(1);
            LCD_Set_Cursor(1, 0); // 調整顯示位址
            delay_ms(1);

            /* btoa no support at XC8, to use sprintf replace it. , Adam Syu */
            sprintf(DisplayString, "%02d:%02d:%02d", hour, min, sec); // 顯示時間資料
            putsLCD(DisplayString);


            //EESequentialRead(I2C_TCN75A_address, 0x00, Temp, 2); //讀取溫度值
            LCD_Set_Cursor(1, 9); // 調整顯示位址

            /* btoa no support at XC8, to use sprintf replace it. , Adam Syu */
            sprintf(DisplayString, "%2d", Temp[0]); // 顯示時間資料
            putsLCD(DisplayString);
            putcLCD(0xDF);
            putcLCD('C');

            if (FLAGbits.TxD)
            {
                putrsUSART("Welcome to PIC! ");
                FLAGbits.TxD = 0;
            }

            FLAGbits.update = 0; // 清除更新旗標
        }
    }
}
// 時間延遲函式

void delay_ms(long A)
{
    //This function is only good for OSC_CLOCK higher than 4MHz
    long i;
    int us2TCY;
    us2TCY = (10 * OSC_CLOCK) >> 2;
    for (i = 0; i < A; i++) Delay100TCYx(us2TCY);
}

void rot_right(void)
{
    if (PORTD > 1) // PORTD>1，向右移動
        PORTD = (PORTD >> 1);
    else // PORTD=1，回歸至RD7
        PORTD = 0x80;
}

void rot_left(void)
{
    if (PORTD < 128) // PORTD<128，向左移動
        PORTD = (PORTD << 1);
    else // PORTD>=128，回歸至RD0
        PORTD = 0X01;
}

//***********************************************
// Put a byte in decimal format to LCD Module
//0~99 Only. Uncomment the first 2 lines for hundreds

void put_Num_LCD(unsigned char The_Number)
{

    unsigned char Temp_Char_100, Temp_Char_10, Temp_Char;

    Temp_Char_100 = The_Number / 100; //取百位數的數字
    putcLCD(Temp_Char_100 + '0'); //轉換為ASCII編碼，'0'的編碼加數字大小
    Temp_Char_10 = (The_Number - Temp_Char_100 * 100) / 10; //取十位數的數字
    putcLCD(Temp_Char_10 + '0');
    Temp_Char = The_Number - (Temp_Char_100 * 100 + Temp_Char_10 * 10); //取個位數的數字
    putcLCD(Temp_Char + '0');
}
